import time

from pytagsfs.debug import log_critical, log_info
from pytagsfs.util import wraps


_profiling_enabled = False

def enable_profiling():
    log_info('enabling profiling; times are in ms')
    global _profiling_enabled
    _profiling_enabled = True


def disable_profiling():
    log_info('disabling profiling')
    global _profiling_enabled
    _profiling_enabled = False


def profile(fn, args, parts):
    start = int(1000.0 * time.time())
    try:
        return fn(*args)
    finally:
        duration = int(1000.0 * time.time()) - start
        parts.insert(0, unicode(duration))
        log_critical(u'PROF %s' % ' '.join(parts))


def profiled_fs_method(fn):
    @wraps(fn)
    def wrapped(self, *args):
        if not _profiling_enabled:
            return fn(self, *args)
        parts = [fn.__name__]
        parts.extend([unicode(a) for a in args])
        return profile(fn, (self,) + args, parts)
    return wrapped


def profiled_file_method(fn):
    @wraps(fn)
    def wrapped(self, *args):
        if not _profiling_enabled:
            return fn(self, *args)
        parts = [fn.__name__, self.fake_path]
        parts.extend([unicode(a) for a in args])
        return profile(fn, (self,) + args, parts)
    return wrapped


def profiled_file_method_noargs(fn):
    @wraps(fn)
    def wrapped(self, *args):
        if not _profiling_enabled:
            return fn(self, *args)
        parts = [fn.__name__, self.fake_path]
        return profile(fn, (self,) + args, parts)
    return wrapped
