import os
import fuse

from pytagsfs.util import return_errno

def delegated_class_method(name):
    def fn(self, fake_path, *args, **kwargs):
        try:
            cls = self.special_file_classes[fake_path[1:]]
        except KeyError:
            return getattr(super(SpecialFileFuseMixin, self), name)(
              fake_path, *args, **kwargs)
        return getattr(cls, name)(fake_path, *args, **kwargs)
    return fn

class SpecialFileFuseMixin(object):
    special_file_classes = None
    special_file_entries = None

    def __init__(self, *args, **kwargs):
        if self.special_file_entries is None:
            self.special_file_entries = [fuse.Direntry(e) for e in list(
              self.special_file_classes.keys())]
        super(SpecialFileFuseMixin, self).__init__(*args, **kwargs)

    def fsinit(self):
        for k in self.special_file_classes:
            self.special_file_classes[k].filesystem = self
        return super(SpecialFileFuseMixin, self).fsinit()

    truncate = return_errno(delegated_class_method('truncate'))
    open = return_errno(delegated_class_method('open'))
    release = return_errno(delegated_class_method('release'))
    create = return_errno(delegated_class_method('create'))
    utime = return_errno(delegated_class_method('utime'))
    access = return_errno(delegated_class_method('access'))
    rename = return_errno(delegated_class_method('rename'))
    mkdir = return_errno(delegated_class_method('mkdir'))
    rmdir = return_errno(delegated_class_method('rmdir'))

    _getattr = delegated_class_method('getattr')

    @return_errno
    def getattr(self, fake_path):
        stat_result = self._getattr(fake_path)
        if fake_path == os.path.sep:
            try:
                stat_result_args = list(stat_result)
            except TypeError:
                return stat_result
            stat_result_args[3] = (
              stat_result_args[3] + len(self.special_file_entries))
            stat_result = os.stat_result(stat_result_args)
        return stat_result

    _readdir = delegated_class_method('readdir')

    @return_errno
    def readdir(self, fake_path, offset):
        result = self._readdir(fake_path, offset)
        if fake_path == os.path.sep:
            new_result = list(self.special_file_entries)
            new_result.extend(result)
            result = new_result
        return result

def not_implemented(*args, **kwargs):
    raise NotImplementedError

class SpecialFile(object):
    filesystem = None

    truncate = classmethod(return_errno(not_implemented))
    open = classmethod(return_errno(not_implemented))
    release = classmethod(return_errno(not_implemented))
    create = classmethod(return_errno(not_implemented))
    utime = classmethod(return_errno(not_implemented))
    access = classmethod(return_errno(not_implemented))
    rename = classmethod(return_errno(not_implemented))
    mkdir = classmethod(return_errno(not_implemented))
    rmdir = classmethod(return_errno(not_implemented))
    getattr = classmethod(return_errno(not_implemented))
    readdir = classmethod(return_errno(not_implemented))
